import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtWebEngine

import "../components"
import "view"

import Artix.Handlers
import Artix.Core
import Artix.Gui

/**
 * @brief Форма web-браузера
 */
BasicView {
    id: root
    readonly property bool fullScreen: true

    implicitWidth: Screen.desktopAvailableWidth
    implicitHeight: Screen.desktopAvailableHeight

    header: Item {
        height: 72
        Item {
            anchors { top: parent.top; bottom: parent.bottom; left: parent.left }
            width: parent.width - topPanel.iconsWidth - 16
            RowLayout {
                x: 16; width: parent.width - x; height: parent.height; spacing: 24
                // Закрытие диалога
                BasicButton {
                    Style.class: "webBrowserExit"
                    Layout.preferredWidth: 56; Layout.preferredHeight: 56; radius: 28
                    Layout.alignment: Qt.AlignVCenter
                    icon: Style.parameter("icon.browser.exit")
                    onClicked: handler.emitAction("DIALOG_WEB_BROWSER")
                }
                // Перейти на предыдущую страницу
                BasicButton {
                    Style.class: "webBrowserCommon"
                    Layout.preferredWidth: 56; Layout.preferredHeight: 56; radius: 28
                    Layout.alignment: Qt.AlignVCenter
                    enabled: webView.canGoBack
                    icon: webView.canGoBack ? Style.parameter("icon.browser.back")
                                            : Style.parameter("icon.browser.backDisabled")
                    onClicked: webView.goBack()
                }
                // Перейти на следующую страницу
                BasicButton {
                    Style.class: "webBrowserCommon"
                    Layout.preferredWidth: 56; Layout.preferredHeight: 56; radius: 28
                    Layout.alignment: Qt.AlignVCenter
                    enabled: webView.canGoForward
                    icon: webView.canGoForward ? Style.parameter("icon.browser.forward")
                                               : Style.parameter("icon.browser.forwardDisabled")
                    onClicked: webView.goForward()
                }
                // Обновить страницу или остановить загрузку страницы
                BasicButton {
                    Style.class: "webBrowserCommon"
                    Layout.preferredWidth: 56; Layout.preferredHeight: 56; radius: 28
                    Layout.alignment: Qt.AlignVCenter
                    icon: webView.loading ? Style.parameter("icon.browser.stop")
                                          : Style.parameter("icon.browser.reload")
                    onClicked: {
                        if (webView.loading)
                            webView.stop()
                        else
                            webView.reload()
                    }
                }
                // Заголовок страницы
                BasicLabel {
                    Style.class: "webBrowserTitle"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 24
                    Layout.alignment: Qt.AlignVCenter
                    text: webView.title
                    elide: Label.ElideRight
                }
            }
        }
        TopPanel {
            id: topPanel
            anchors { top: parent.top; bottom: parent.bottom; right: parent.right }
            hideUserName: true
        }
        // Прогресс загрузки страницы
        ProgressBar {
            anchors.top: parent.bottom
            width: parent.width; height: 2
            to: 100
            value: webView.loadProgress
            visible: webView.loading
            contentItem: Item {
                property real progressWidth: width * parent.visualPosition
                Rectangle {
                    x: 0; y: 0
                    width: parent.progressWidth; height: parent.height
                    color: Style.parameter("color.blueDark")
                }
            }
            background: null
        }
    }

    contentItem: WebEngineView {
        id: webView
        url: params.url
        settings.defaultTextEncoding: params.codePage
    }

    footer: BottomPanel { id: bottomPanel }

    DialogHandler { id: handler }

    // Всплывающая клавиатура
    Popup {
        id: keyboard
        parent: Overlay.overlay
        x: 0; y: root.height
        width: root.width; height: root.width * 0.402; padding: 32
        leftPadding: params.showDigitalKeyboard ? width * 0.33 : 32
        rightPadding: params.showDigitalKeyboard ? width * 0.33 : 32
        closePolicy: Popup.NoAutoClose
        contentItem: InlineKeyboard {
            layoutType: params.showDigitalKeyboard ? "input_digits" : "main"
            onEnter: handler.emitKey(Qt.Key_Enter)
        }
        background: Rectangle {
            color: Style.parameter("color.modalBackground")
            // Тень
            Rectangle {
                anchors.bottom: parent.top
                width: parent.width; height: 5
                gradient: Gradient {
                    orientation: Gradient.Vertical
                    GradientStop { position: 0.0; color: "transparent" }
                    GradientStop { position: 1.0; color: Style.parameter("color.shadow") }
                }
                opacity: 0.3
            }
        }
        Connections {
            target: Qt.inputMethod
            function onVisibleChanged() {
                if (!params.showKeyboard)
                    return

                if (Qt.inputMethod.visible)
                    keyboard.open()
                else
                    keyboard.close()
            }
        }
        enter: Transition { SmoothedAnimation { velocity: 1500; property: "y"; to: root.height - keyboard.height - bottomPanel.height } }
        exit: Transition { SmoothedAnimation { velocity: 1500; property: "y"; to: root.height } }
    }

    LockDelay { enabled: params.lockDelay }
    EventFilterConnector {
        filter: BrowserEventFilter {
            blockAllKeyEvents: false
            onExit: handler.emitAction("DIALOG_WEB_BROWSER")
        }
    }
}
